<?php

namespace Drupal\node_boolean\Plugin\Condition;

use Drupal\Core\Condition\ConditionPluginBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\Context\ContextDefinition;

/**
 * Provides a 'Node boolean' condition to enable a condition based in module selected status.
 *
 * @Condition(
 *   id = "node_boolean",
 *   label = @Translation("Node boolean"),
 *   context = {
 *     "node" = @ContextDefinition("entity:node", required = TRUE , label = @Translation("node"))
 *   }
 * )
 *
 */
class NodeBoolean extends ConditionPluginBase {
  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition)
  {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition
    );
  }

  /**
   * Creates a new ExampleCondition instance.
   *
   * @param array $configuration
   *   The plugin configuration, i.e. an array with configuration values keyed
   *   by configuration option name. The special key 'context' may be used to
   *   initialize the defined contexts by setting it to an array of context
   *   values keyed by context names.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $options = array();
    foreach($this->getBooleanNodeFieldMap() as $field_id => $field) {
      $options[$field_id] = ucfirst(str_replace('_', ' ', $field_id));
    }

    $form['boolean'] = array(
      '#type' => 'checkboxes',
      '#title' => $this->t('Fields that should be true (any)'),
      '#default_value' => $this->configuration['boolean'],
      '#options' => $options,
    );

    $form['all'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Evaluate <b>all</b> fields rather than <b>any</b>'),
      '#default_value' => $this->configuration['all'],
    );

    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['boolean'] = array_filter($form_state->getValue('boolean'));
    $this->configuration['all'] = $form_state->getValue('all');
    parent::submitConfigurationForm($form, $form_state);

    if (!$this->configuration['boolean']) {
      $this->configuration = [];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return array('boolean' => array(), 'all' => false) + parent::defaultConfiguration();
  }

  /**
   * Evaluates the condition and returns TRUE or FALSE accordingly.
   *
   * @return bool
   *   TRUE if the condition has been met, FALSE otherwise.
   */
  public function evaluate() {
    if (empty($this->configuration['boolean']) && !$this->isNegated()) {
      return TRUE;
    }
    if (isset($this->configuration['all']) && $this->configuration['all']) {
      return $this->evaluateAll();
    } else {
      return $this->evaluateAny();
    }
  }

  /**
   * Evaluates the condition and returns TRUE or FALSE accordingly for any fields.
   *
   * @return bool
   *   TRUE if the condition has been met, FALSE otherwise.
   */
  protected function evaluateAny() {
    $fields = $this->getBooleanNodeFieldMap();
    $node = $this->getContextValue('node');
    foreach ($this->configuration['boolean'] as $field_id) {
      if (isset($fields[$field_id]['bundles'][$node->getType()])) {
        $fieldValue = FALSE;
        if ($node->get($field_id)->count()) {
          $fieldValue = $node->get($field_id)->first()->get('value')->getValue();
        }
        if ($fieldValue) {
          return TRUE;
        } else if ($this->isNegated()) {
          return FALSE;
        }
      }
    }
    return FALSE;
  }

  /**
   * Evaluates the condition and returns TRUE or FALSE accordingly for ALL fields.
   *
   * @return bool
   *   TRUE if the condition has been met, FALSE otherwise.
   */
  protected function evaluateAll() {
    $fields = $this->getBooleanNodeFieldMap();
    $node = $this->getContextValue('node');
    foreach ($this->configuration['boolean'] as $field_id) {
      if (isset($fields[$field_id]['bundles'][$node->getType()])) {
        $fieldValue = FALSE;
        if ($node->get($field_id)->count()) {
          $fieldValue = $node->get($field_id)->first()->get('value')->getValue();
        }
        if ((!$fieldValue && !$this->isNegated()) || ($fieldValue && $this->isNegated())) {
          return TRUE;
        }
      }
    }
    return FALSE;
  }

  /**
   * Provides a human readable summary of the condition's configuration.
   */
  public function summary()
  {
    $booleans = $this->configuration['boolean'];
    return t('One of the following fields must evaluate to true: @fields.', array('@fields' => explode(', ', $booleans)));
  }

  /**
   * Provides a lightweight map of node fields of the boolean type.
   */
  protected function getBooleanNodeFieldMap()
  {
    $fields = \Drupal::service('entity_field.manager')->getFieldMapByFieldType('boolean');
    return $fields['node'];
  }

}
